#
# SecBus ( http://secbus.telecom-paristech.fr/ ) - This file is part of SecBus
# Copyright (C) - Telecom ParisTech
# Contacts: contact-secbus@telecom-paristech.fr
#
# This file must be used under the terms of the CeCILL.
# This source file is licensed as described in the file COPYING, which
# you should have received as part of this distribution. The terms
# are also available at
# http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
#

#############
# Create IP #
#############

create_project ip . -part xc7z020clg484-1
set_property board_part em.avnet.com:zed:part0:0.9 [current_project]
add_files ../global.vhd ../axi_pkg.vhd ../bitfield_pkg.vhd ../uart2maxilite_pkg.vhd ../uart2maxilite.vhd
# set_property library $l [ get_files $p/$d ]
import_files -force -norecurse
update_compile_order -fileset sources_1
ipx::package_project -root_dir ./ip.srcs/sources_1/imports
set_property vendor {www.telecom-paristech.fr} [ipx::current_core]
set_property library {SecBus} [ipx::current_core]
ipx::update_checksums [ipx::current_core]
ipx::save_core [ipx::current_core]
set_property ip_repo_paths ./ip.srcs/sources_1/imports [current_fileset]
update_ip_catalog
ipx::check_integrity -quiet [ipx::current_core]
ipx::archive_core ./ip.srcs/sources_1/imports/www.telecom-paristech.fr_SecBus_ip_1.0.zip [ipx::current_core]
close_project

##################
## Create design #
##################

create_project top . -part xc7z020clg484-1
set_property board_part em.avnet.com:zed:part0:0.9 [current_project]
set_property ip_repo_paths . [current_fileset]
update_ip_catalog
create_bd_design "top"
set u2m [create_bd_cell -type ip -vlnv www.telecom-paristech.fr:SecBus:uart2maxilite:1.0 ip_0]
set ps7 [create_bd_cell -type ip -vlnv xilinx.com:ip:processing_system7:5.5 processing_system7_0]
set uart [create_bd_cell -type ip -vlnv xilinx.com:ip:axi_uartlite:2.0 axi_uartlite_0]
apply_bd_automation -rule xilinx.com:bd_rule:processing_system7 -config {make_external "FIXED_IO, DDR" apply_board_preset "1" Master "Disable" Slave "Disable" } $ps7
set_property -dict [list CONFIG.PCW_FPGA0_PERIPHERAL_FREQMHZ {100.000000}] $ps7
set_property -dict [list CONFIG.PCW_USE_M_AXI_GP0 {0}] $ps7
set_property -dict [list CONFIG.PCW_USE_M_AXI_GP1 {0}] $ps7
set_property -dict [list CONFIG.PCW_USE_S_AXI_HP0 {0}] $ps7
set_property -dict [list CONFIG.PCW_USE_S_AXI_HP1 {1}] $ps7
set_property -dict [list CONFIG.PCW_S_AXI_HP1_DATA_WIDTH {32}] $ps7
set_property -dict [list CONFIG.PARITY {No_Parity}] $uart
set_property -dict [list CONFIG.C_BAUDRATE {115200}] $uart
set_property -dict [list CONFIG.C_DATA_BITS {8}] $uart

# Interconnections
# Primary IOs
create_bd_port -dir O -from 7 -to 0 gpo
connect_bd_net [get_bd_pins /ip_0/gpo] [get_bd_ports gpo]
create_bd_port -dir I -from 7 -to 0 gpi
connect_bd_net [get_bd_pins /ip_0/gpi] [get_bd_ports gpi]
create_bd_port -dir I srst
connect_bd_net [get_bd_pins /ip_0/srst] [get_bd_ports srst]
create_bd_port -dir I -type data rx
connect_bd_net [get_bd_ports rx] [get_bd_pins axi_uartlite_0/rx]
create_bd_port -dir O -type data tx
connect_bd_net [get_bd_ports tx] [get_bd_pins axi_uartlite_0/tx]
# ps7 - ip
apply_bd_automation -rule xilinx.com:bd_rule:axi4 -config {Master "/ip_0/mem_axi" Clk "Auto" }  [get_bd_intf_pins processing_system7_0/S_AXI_HP1]
# uart - ip
connect_bd_net [get_bd_pins axi_uartlite_0/rx] [get_bd_pins ip_0/rx]
connect_bd_net [get_bd_pins axi_uartlite_0/tx] [get_bd_pins ip_0/tx]
apply_bd_automation -rule xilinx.com:bd_rule:axi4 -config {Master "/ip_0/uart_axi" Clk "Auto" }  [get_bd_intf_pins axi_uartlite_0/S_AXI]

# Addresses ranges
set_property offset 0x00000000 [get_bd_addr_segs [list ip_0/mem_axi/SEG_processing_system7_0_HP1_DDR_LOWOCM]]
set_property range 512M [get_bd_addr_segs {ip_0/mem_axi/SEG_processing_system7_0_HP1_DDR_LOWOCM}]
set_property offset 0x000 [get_bd_addr_segs [list ip_0/uart_axi/SEG_axi_uartlite_0_Reg]]
set_property range 4K [get_bd_addr_segs [list ip_0/uart_axi/SEG_axi_uartlite_0_Reg]]

# Synthesis flow
validate_bd_design
generate_target all [get_files top.srcs/sources_1/bd/top/top.bd]
make_wrapper -files [get_files top.srcs/sources_1/bd/top/top.bd] -top
add_files -norecurse -force top.srcs/sources_1/bd/top/hdl/top_wrapper.v
update_compile_order -fileset sources_1
update_compile_order -fileset sim_1
save_bd_design
set_property STEPS.SYNTH_DESIGN.ARGS.FLATTEN_HIERARCHY none [get_runs synth_1]
set_property STEPS.SYNTH_DESIGN.ARGS.DIRECTIVE RuntimeOptimized [get_runs synth_1]
launch_runs synth_1
wait_on_run synth_1
open_run synth_1 -name netlist_1

# IOs
array set ios {
	"gpi[0]"	{ "F22"  "LVCMOS25" }
	"gpi[1]"	{ "G22"  "LVCMOS25" }
	"gpi[2]"	{ "H22"  "LVCMOS25" }
	"gpi[3]"	{ "F21"  "LVCMOS25" }
	"gpi[4]"	{ "H19"  "LVCMOS25" }
	"gpi[5]"	{ "H18"  "LVCMOS25" }
	"gpi[6]"	{ "H17"  "LVCMOS25" }
	"gpi[7]"	{ "M15"  "LVCMOS25" }
	"gpo[0]"	{ "T22"  "LVCMOS33" }
	"gpo[1]"	{ "T21"  "LVCMOS33" }
	"gpo[2]"	{ "U22"  "LVCMOS33" }
	"gpo[3]"	{ "U21"  "LVCMOS33" }
	"gpo[4]"	{ "V22"  "LVCMOS33" }
	"gpo[5]"	{ "W22"  "LVCMOS33" }
	"gpo[6]"	{ "U19"  "LVCMOS33" }
	"gpo[7]"	{ "U14"  "LVCMOS33" }
	"srst"		{ "P16"  "LVCMOS25" }
	"tx"		{ "AA11" "LVCMOS33" }
	"rx"		{ "Y10"  "LVCMOS33" }
}
foreach io [ array names ios ] {
	set pin [ lindex $ios($io) 0 ]
	set std [ lindex $ios($io) 1 ]
	set_property package_pin $pin [get_ports $io]
	set_property iostandard $std [get_ports [list $io]]
}

# Timing constraints
set_false_path -from [get_clocks {clk_fpga_0}] -to [get_ports {gpo[*] rx}]
set_false_path -from [get_ports {gpi[*] tx srst}] -to [get_clocks {clk_fpga_0}]

save_bd_design
save_constraints
reset_run impl_1
set_property STEPS.OPT_DESIGN.ARGS.DIRECTIVE RuntimeOptimized [get_runs impl_1]
set_property STEPS.PLACE_DESIGN.ARGS.DIRECTIVE RuntimeOptimized [get_runs impl_1]
set_property STEPS.ROUTE_DESIGN.ARGS.DIRECTIVE RuntimeOptimized [get_runs impl_1]
set_property STEPS.WRITE_BITSTREAM.ARGS.BIN_FILE true [get_runs impl_1]
launch_runs impl_1 -to_step write_bitstream
wait_on_run impl_1
