--
-- SecBus ( http://secbus.telecom-paristech.fr/ ) - This file is part of SecBus
-- Copyright (C) - Telecom ParisTech
-- Contacts: contact-secbus@telecom-paristech.fr
--
-- This file must be used under the terms of the CeCILL.
-- This source file is licensed as described in the file COPYING, which
-- you should have received as part of this distribution. The terms
-- are also available at
-- http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.bitfield_pkg.all;
use work.uart2maxilite_pkg.all;
use work.axi_pkg.all;

entity uart2maxilite is
  port(
    aclk:       in std_logic;
    aresetn:    in std_logic;
    srst:       in std_logic;
    ---------------------------------
    -- AXI lite master port uart_axi --
    ---------------------------------
    -- Outputs (master to slave) --
    -------------------------------
    -- Read address channel
    uart_axi_araddr:  out std_logic_vector(uart_na - 1 downto 0);
    uart_axi_arprot:  out std_logic_vector(2 downto 0);
    uart_axi_arvalid: out std_logic;
    -- Read data channel
    uart_axi_rready:  out std_logic;
    -- Write address channel
    uart_axi_awaddr:  out std_logic_vector(uart_na - 1 downto 0);
    uart_axi_awprot:  out std_logic_vector(2 downto 0);
    uart_axi_awvalid: out std_logic;
    -- Write data channel
    uart_axi_wdata:   out std_logic_vector(31 downto 0);
    uart_axi_wstrb:   out std_logic_vector(3 downto 0);
    uart_axi_wvalid:  out std_logic;
    -- Write response channel
    uart_axi_bready:  out std_logic;
    ------------------------------
    -- Inputs (slave to master) --
    ------------------------------
    -- Read address channel
    uart_axi_arready: in  std_logic;
    -- Read data channel
    uart_axi_rdata:   in  std_logic_vector(31 downto 0);
    uart_axi_rresp:   in  std_logic_vector(1 downto 0);
    uart_axi_rvalid:  in  std_logic;
    -- Write address channel
    uart_axi_awready: in  std_logic;
    -- Write data channel
    uart_axi_wready:  in  std_logic;
    -- Write response channel
    uart_axi_bvalid:  in  std_logic;
    uart_axi_bresp:   in  std_logic_vector(1 downto 0);

    ---------------------------------
    -- AXI lite master port mem_axi --
    ---------------------------------
    -- Outputs (master to slave) --
    -------------------------------
    -- Read address channel
    mem_axi_araddr:  out std_logic_vector(31 downto 0);
    mem_axi_arprot:  out std_logic_vector(2 downto 0);
    mem_axi_arvalid: out std_logic;
    -- Read data channel
    mem_axi_rready:  out std_logic;
    -- Write address channel
    mem_axi_awaddr:  out std_logic_vector(31 downto 0);
    mem_axi_awprot:  out std_logic_vector(2 downto 0);
    mem_axi_awvalid: out std_logic;
    -- Write data channel
    mem_axi_wdata:   out std_logic_vector(31 downto 0);
    mem_axi_wstrb:   out std_logic_vector(3 downto 0);
    mem_axi_wvalid:  out std_logic;
    -- Write response channel
    mem_axi_bready:  out std_logic;
    ------------------------------
    -- Inputs (slave to master) --
    ------------------------------
    -- Read address channel
    mem_axi_arready: in  std_logic;
    -- Read data channel
    mem_axi_rdata:   in  std_logic_vector(31 downto 0);
    mem_axi_rresp:   in  std_logic_vector(1 downto 0);
    mem_axi_rvalid:  in  std_logic;
    -- Write address channel
    mem_axi_awready: in  std_logic;
    -- Write data channel
    mem_axi_wready:  in  std_logic;
    -- Write response channel
    mem_axi_bvalid:  in  std_logic;
    mem_axi_bresp:   in  std_logic_vector(1 downto 0);

    -- Debug
    rx, tx:     in std_logic;
    gpi:        in  std_logic_vector(7 downto 0);
    gpo:        out std_logic_vector(7 downto 0)
  );
end entity uart2maxilite;

architecture rtl of uart2maxilite is

  -- Internal, record types, AXI interfaces with UART and MEM
  signal uart_axi_m2s: axilite_gp_m2s;
  signal uart_axi_s2m: axilite_gp_s2m;
  signal mem_axi_m2s:  axilite_gp_m2s;
  signal mem_axi_s2m:  axilite_gp_s2m;

  -- Synchronized version of srst (warning: there is no debouncer, one may be needed)
  signal srst_sync:     std_ulogic;

  -- Local reset (combination of srst and aresetn)
  signal resetn_local: std_ulogic;

  -- Internal registers and fields
  --  COPY-BEGIN
  -----------------------------------------
  --  Auto generated by BFGen, do not edit
  -----------------------------------------
  -- Register set
  signal bf_reg_set: bf_reg_set_t;
  
  -- Register aliases
  alias r_a0 is bf_reg_set(0);
  alias r_a1 is bf_reg_set(1);
  alias r_a2 is bf_reg_set(2);
  alias r_a3 is bf_reg_set(3);
  alias r_d0 is bf_reg_set(4);
  alias r_d1 is bf_reg_set(5);
  alias r_d2 is bf_reg_set(6);
  alias r_d3 is bf_reg_set(7);
  alias r_status is bf_reg_set(8);
  
  -- Field aliases
  alias f_rxvalid is bf_reg_set(8)(0);
  alias f_rxfull is bf_reg_set(8)(1);
  alias f_txempty is bf_reg_set(8)(2);
  alias f_txfull is bf_reg_set(8)(3);
  alias f_ie is bf_reg_set(8)(4);
  alias f_oerr is bf_reg_set(8)(5);
  alias f_ferr is bf_reg_set(8)(6);
  alias f_perr is bf_reg_set(8)(7);
  -- COPY-END

  -- Signals used to launch write and read accesses to UART or MEM
  signal uart_write_go, uart_read_go, mem_write_go, mem_read_go: boolean;

  -- Debuging
  signal beat, rrx, rtx: std_logic;

begin

  -- Default values of some UART AXI interface fields
  uart_axi_m2s.arprot <= (others => '0');
  uart_axi_m2s.rready <= '1';
  uart_axi_m2s.awprot <= (others => '0');
  uart_axi_m2s.wstrb  <= (0 => '1', others => '0');
  uart_axi_m2s.bready <= '1';

  -- Default values of some MEM AXI interface fields
  mem_axi_m2s.araddr <= r_a3 & r_a2 & r_a1 & r_a0;
  mem_axi_m2s.arprot <= (others => '0');
  mem_axi_m2s.rready <= '1';
  mem_axi_m2s.awaddr <= r_a3 & r_a2 & r_a1 & r_a0;
  mem_axi_m2s.awprot <= (others => '0');
  mem_axi_m2s.wdata  <= r_d3 & r_d2 & r_d1 & r_d0;
  mem_axi_m2s.bready <= '1';

  resetn_local <= (not srst_sync) and aresetn;

  -- srst re-synchronizer
  process(aclk)
    variable tmp: std_ulogic_vector(1 downto 0);
  begin
    if rising_edge(aclk) then
      srst_sync <= tmp(0);
      tmp := srst & tmp(1);
    end if;
  end process;

  -- Reader process on UART AXI interface
  uart_read_pr: process(aclk)
    type state_t is (idle, requesting);
    variable state: state_t;
  begin
    if rising_edge(aclk) then
      if aresetn = '0' or srst = '1' then
        uart_axi_m2s.arvalid <= '0';
        state := idle;
      else
        case state is
          when idle =>
            if uart_read_go then
              uart_axi_m2s.arvalid <= '1';
              state := requesting;
            end if;
          when requesting =>
            if uart_axi_s2m.arready = '1' then
              uart_axi_m2s.arvalid <= '0';
              state := idle;
            end if;
        end case;
      end if;
    end if;
  end process uart_read_pr;

  -- Writer process on UART AXI interface
  uart_write_pr: process(aclk)
    type state_t is (idle, requesting);
    variable state: state_t;
  begin
    if rising_edge(aclk) then
      if aresetn = '0' or srst = '1' then
        uart_axi_m2s.awvalid <= '0';
        uart_axi_m2s.wvalid  <= '0';
        state := idle;
      else
        case state is
          when idle =>
            if uart_write_go then
              uart_axi_m2s.awvalid <= '1';
              uart_axi_m2s.wvalid  <= '1';
              state := requesting;
            end if;
          when requesting =>
            if uart_axi_s2m.awready = '1' then
              uart_axi_m2s.awvalid <= '0';
            end if;
            if uart_axi_s2m.wready = '1' then
              uart_axi_m2s.wvalid  <= '0';
            end if;
            if (uart_axi_s2m.awready = '1' or uart_axi_m2s.awvalid = '0') and (uart_axi_s2m.wready = '1' or uart_axi_m2s.wvalid = '0') then
              state := idle;
            end if;
        end case;
      end if;
    end if;
  end process uart_write_pr;

  -- Reader process on MEM AXI interface
  mem_read_pr: process(aclk)
    type state_t is (idle, requesting);
    variable state: state_t;
  begin
    if rising_edge(aclk) then
      if aresetn = '0' or srst = '1' then
        mem_axi_m2s.arvalid <= '0';
        state := idle;
      else
        case state is
          when idle =>
            if mem_read_go then
              mem_axi_m2s.arvalid <= '1';
              state := requesting;
            end if;
          when requesting =>
            if mem_axi_s2m.arready = '1' then
              mem_axi_m2s.arvalid <= '0';
              state := idle;
            end if;
        end case;
      end if;
    end if;
  end process mem_read_pr;

  -- Writer process on MEM AXI interface
  mem_write_pr: process(aclk)
    type state_t is (idle, requesting);
    variable state: state_t;
  begin
    if rising_edge(aclk) then
      if aresetn = '0' or srst = '1' then
        mem_axi_m2s.awvalid <= '0';
        mem_axi_m2s.wvalid  <= '0';
        state := idle;
      else
        case state is
          when idle =>
            if mem_write_go then
              mem_axi_m2s.awvalid <= '1';
              mem_axi_m2s.wvalid  <= '1';
              state := requesting;
            end if;
          when requesting =>
            if mem_axi_s2m.awready = '1' then
              mem_axi_m2s.awvalid <= '0';
            end if;
            if mem_axi_s2m.wready = '1' then
              mem_axi_m2s.wvalid  <= '0';
            end if;
            if (mem_axi_s2m.awready = '1' or mem_axi_m2s.awvalid = '0') and (mem_axi_s2m.wready = '1' or mem_axi_m2s.wvalid = '0') then
              state := idle;
            end if;
        end case;
      end if;
    end if;
  end process mem_write_pr;

  -- Main process
  process(aclk)
    type state_t is (
      uart_init,                            -- Launch write X"03" to UART.ctrl register. Goto uart_wait_write_and_uart_read_c0.
      uart_wait_write_and_uart_read_c0,     -- Wait end of UART write. Goto uart_read_c0.
      uart_read_c0,                         -- Launch read UART.status. Goto uart_read_c0_1.
      uart_read_c0_1,                       -- Wait end of UART read. If RX FIFO not empty go to uart_read_c0_2 else go to uart_read_c0.
      uart_read_c0_2,                       -- Launch read UART.rx. Goto uart_read_c0_3.
      uart_read_c0_3,                       -- Wait end of UART read (C0). Depending on C0:
                                            --   - Launch read MEM. Goto mem_read.
                                            --   - Launch write MEM. Goto mem_write.
                                            --   - Launch write specified register to UART.tx. Goto uart_wait_write_and_uart_read_c0.
                                            --   - Goto uart_read_c1.
      mem_read,                             -- Wait end of MEM read. Launch write status to UART.tx. Goto uart_wait_write_and_uart_read_c0.
      mem_write,                            -- Wait end of MEM write. Launch write status to UART.tx. Goto uart_wait_write_and_uart_read_c0.
      uart_read_c1,                         -- Launch read UART.status. Goto uart_read_c1_1.
      uart_read_c1_1,                       -- Wait end of UART read. If RX FIFO not empty go to uart_read_c1_2 else go to uart_read_c1.
      uart_read_c1_2,                       -- Launch read UART.rx. Goto uart_read_c1_3.
      uart_read_c1_3                        -- Wait end of UART read (C1). Write C1 to specified register. Goto uart_read_c0.
    );

    variable state: state_t;
    variable cmd: std_ulogic_vector(1 downto 0);
    variable status: std_ulogic_vector(7 downto 0);
    variable idx: natural range 0 to 15;
    variable cnt: unsigned(25 downto 0);

  begin
    if rising_edge(aclk) then
      if aresetn = '0' then
        state := uart_init;
      end if;
      if resetn_local = '0' then
        bf_reg_set          <= bf_def;
        uart_axi_m2s.araddr <= (others => '0');
        uart_axi_m2s.awaddr <= (others => '0');
        uart_axi_m2s.wdata  <= (others => '0');
        mem_axi_m2s.wstrb   <= (others => '0');
        cnt   := (others => '0');
        beat  <= '0';
        rrx   <= '1';
        rtx   <= '1';
      else
        cnt  := cnt + 1;
        beat <= cnt(25);
        rrx  <= rx and rrx;
        rtx  <= tx and rtx;
        uart_write_go <= false;
        uart_read_go  <= false;
        mem_write_go  <= false;
        mem_read_go   <= false;
        case state is
          when uart_init =>
            uart_axi_m2s.awaddr <= uart_ctrl_addr;
            uart_axi_m2s.wdata(uart_ctrl_rst_tx_fifo) <= '1';
            uart_axi_m2s.wdata(uart_ctrl_rst_rx_fifo) <= '1';
            uart_write_go <= true;
            state := uart_wait_write_and_uart_read_c0;
          when uart_wait_write_and_uart_read_c0 =>
            if uart_axi_s2m.bvalid = '1' then
              state := uart_read_c0;
            end if;
          when uart_read_c0 =>
            uart_axi_m2s.araddr <= uart_status_addr;
            uart_read_go <= true;
            state := uart_read_c0_1;
          when uart_read_c0_1 =>
            if uart_axi_s2m.rvalid = '1' then
              r_status <= uart_axi_s2m.rdata;
              r_status(7 downto 5) <= uart_axi_s2m.rdata(7 downto 5) or r_status(7 downto 5);
              r_status(3) <= uart_axi_s2m.rdata(3) or r_status(3);
              if uart_axi_s2m.rdata(uart_status_rx_fifo_valid_data) /= '1' then
                state := uart_read_c0;
              else
                state := uart_read_c0_2;
              end if;
            end if;
          when uart_read_c0_2 =>
            uart_axi_m2s.araddr <= uart_rx_addr;
            uart_read_go <= true;
            state := uart_read_c0_3;
          when uart_read_c0_3 =>
            if uart_axi_s2m.rvalid = '1' then
              cmd := uart_axi_s2m.rdata(5 downto 4);
              mem_axi_m2s.wstrb <= uart_axi_s2m.rdata(3 downto 0);
              status := okay_char;
              case cmd is
                when cmd_mem_read =>
                  mem_read_go <= true;
                  state := mem_read;
                when cmd_mem_write =>
                  mem_write_go <= true;
                  state := mem_write;
                when cmd_reg_read =>
                  idx := to_integer(unsigned(uart_axi_s2m.rdata(2 downto 0)));
                  uart_axi_m2s.wdata(7 downto 0) <= bf_reg_set(idx);
                  uart_axi_m2s.awaddr <= uart_tx_addr;
                  uart_write_go <= true;
                  state := uart_wait_write_and_uart_read_c0;
                when others =>
                  idx := to_integer(unsigned(uart_axi_s2m.rdata(2 downto 0)));
                  state := uart_read_c1;
              end case;
            end if;
          when mem_read =>
            if mem_axi_s2m.rvalid = '1' then
              if mem_axi_s2m.rresp = axi_resp_okay then
                r_d3 <= mem_axi_s2m.rdata(31 downto 24);
                r_d2 <= mem_axi_s2m.rdata(23 downto 16);
                r_d1 <= mem_axi_s2m.rdata(15 downto 8);
                r_d0 <= mem_axi_s2m.rdata(7 downto 0);
              elsif mem_axi_s2m.rresp = axi_resp_slverr then
                status := slverr_char;
              else
                status := decerr_char;
              end if;
              uart_axi_m2s.awaddr <= uart_tx_addr;
              uart_axi_m2s.wdata(7 downto 0) <= status;
              uart_write_go <= true;
              state := uart_wait_write_and_uart_read_c0;
            end if;
          when mem_write =>
            if mem_axi_s2m.bvalid = '1' then
              if mem_axi_s2m.bresp = axi_resp_slverr then
                status := slverr_char;
              elsif mem_axi_s2m.bresp = axi_resp_decerr then
                status := decerr_char;
              end if;
              uart_axi_m2s.awaddr <= uart_tx_addr;
              uart_axi_m2s.wdata(7 downto 0) <= status;
              uart_write_go <= true;
              state := uart_wait_write_and_uart_read_c0;
            end if;
          when uart_read_c1 =>
            uart_axi_m2s.araddr <= uart_status_addr;
            uart_read_go <= true;
            state := uart_read_c1_1;
          when uart_read_c1_1 =>
            if uart_axi_s2m.rvalid = '1' then
              r_status <= uart_axi_s2m.rdata;
              r_status(7 downto 5) <= uart_axi_s2m.rdata(7 downto 5) or r_status(7 downto 5);
              r_status(3) <= uart_axi_s2m.rdata(3) or r_status(3);
              if uart_axi_s2m.rdata(uart_status_rx_fifo_valid_data) /= '1' then
                state := uart_read_c1;
              else
                state := uart_read_c1_2;
              end if;
            end if;
          when uart_read_c1_2 =>
            uart_axi_m2s.araddr <= uart_rx_addr;
            uart_read_go <= true;
            state := uart_read_c1_3;
          when uart_read_c1_3 =>
            if uart_axi_s2m.rvalid = '1' then
              bf_reg_set(idx) <= uart_axi_s2m.rdata(7 downto 0);
              state := uart_read_c0;
            end if;
        end case;
      end if;
    end if;
  end process;

  uart_axi_araddr       <= std_logic_vector(uart_axi_m2s.araddr(uart_na - 1 downto 0));
  uart_axi_arprot       <= std_logic_vector(uart_axi_m2s.arprot);
  uart_axi_arvalid      <= uart_axi_m2s.arvalid;
  uart_axi_rready       <= uart_axi_m2s.rready;
  uart_axi_awaddr       <= std_logic_vector(uart_axi_m2s.awaddr(uart_na - 1 downto 0));
  uart_axi_awprot       <= std_logic_vector(uart_axi_m2s.awprot);
  uart_axi_awvalid      <= uart_axi_m2s.awvalid;
  uart_axi_wdata        <= std_logic_vector(uart_axi_m2s.wdata);
  uart_axi_wstrb        <= std_logic_vector(uart_axi_m2s.wstrb);
  uart_axi_wvalid       <= uart_axi_m2s.wvalid;
  uart_axi_bready       <= uart_axi_m2s.bready;
  uart_axi_s2m.arready  <= uart_axi_arready;
  uart_axi_s2m.rdata    <= std_ulogic_vector(uart_axi_rdata);
  uart_axi_s2m.rresp    <= std_ulogic_vector(uart_axi_rresp);
  uart_axi_s2m.rvalid   <= uart_axi_rvalid;
  uart_axi_s2m.awready  <= uart_axi_awready;
  uart_axi_s2m.wready   <= uart_axi_wready;
  uart_axi_s2m.bvalid   <= uart_axi_bvalid;
  uart_axi_s2m.bresp    <= std_ulogic_vector(uart_axi_bresp);

  mem_axi_araddr       <= std_logic_vector(mem_axi_m2s.araddr);
  mem_axi_arprot       <= std_logic_vector(mem_axi_m2s.arprot);
  mem_axi_arvalid      <= mem_axi_m2s.arvalid;
  mem_axi_rready       <= mem_axi_m2s.rready;
  mem_axi_awaddr       <= std_logic_vector(mem_axi_m2s.awaddr);
  mem_axi_awprot       <= std_logic_vector(mem_axi_m2s.awprot);
  mem_axi_awvalid      <= mem_axi_m2s.awvalid;
  mem_axi_wdata        <= std_logic_vector(mem_axi_m2s.wdata);
  mem_axi_wstrb        <= std_logic_vector(mem_axi_m2s.wstrb);
  mem_axi_wvalid       <= mem_axi_m2s.wvalid;
  mem_axi_bready       <= mem_axi_m2s.bready;
  mem_axi_s2m.arready  <= mem_axi_arready;
  mem_axi_s2m.rdata    <= std_ulogic_vector(mem_axi_rdata);
  mem_axi_s2m.rresp    <= std_ulogic_vector(mem_axi_rresp);
  mem_axi_s2m.rvalid   <= mem_axi_rvalid;
  mem_axi_s2m.awready  <= mem_axi_awready;
  mem_axi_s2m.wready   <= mem_axi_wready;
  mem_axi_s2m.bvalid   <= mem_axi_bvalid;
  mem_axi_s2m.bresp    <= std_ulogic_vector(mem_axi_bresp);


  process(gpi, bf_reg_set, aresetn, srst, resetn_local, beat, rrx, rtx)
    variable tmp: natural range 0 to 255;
  begin
    tmp := to_integer(unsigned(gpi));
    if tmp < bf_num_reg then
      gpo <= std_logic_vector(bf_reg_set(tmp));
    else
      gpo <= "00" & aresetn & srst & resetn_local & beat & rrx & rtx;
    end if;
  end process;

end architecture rtl;
